@@ -26,45 +26,41 @@ class Sample:
         Modifies list of N cell states into N lists of 1/0 state for each cell
         """
         for cell_type1 in list(set(self.cell_types)):
-            list_ = []
+            states = []
             for cell_type2 in self.cell_types:
                 if cell_type2 == cell_type1:
-                    list_.append(1)
+                    states.append(1)
                 else:
-                    list_.append(0)
-            self.cell_types_df[cell_type1] = list_
+                    states.append(0)
+            self.cell_types_df[cell_type1] = states
 
     def generate_model(self, cluster_col):
         """
-        Generates LGBM model for a given Y
+        Generates Light GBM model for a given Y
         """
-        X_train, X_test, y_train, y_test = train_test_split(self.matrix,
-                                                            cluster_col,
-                                                            test_size=0.25,
-                                                            random_state=42)
+        x_train, x_test, \
+            y_train, y_test = train_test_split(self.matrix, cluster_col, test_size=0.25, random_state=42)
         model = LGBMRegressor(n_estimators=500, first_metric_only=True)
-        model.fit(X_train, y_train,
-                  eval_set=[(X_test, y_test)],
+        model.fit(x_train, y_train,
+                  eval_set=[(x_test, y_test)],
                   eval_metric=['auc'],
                   early_stopping_rounds=10,
                   verbose=0)
-        top_features = pd.DataFrame(
-            [X_train.columns,
-             model.feature_importances_]).T.sort_values(1, ascending=False)
-        X_train_upd = sigmoid(update_df(X_train, list(top_features[0][:3000])))
-        X_test_upd = sigmoid(update_df(X_test, list(top_features[0][:3000])))
+        top_features = pd.DataFrame([x_train.columns, model.feature_importances_]).T.sort_values(
+            1, ascending=False)
+        x_train_upd = sigmoid(update_df(x_train, list(top_features[0][:3000])))
+        x_test_upd = sigmoid(update_df(x_test, list(top_features[0][:3000])))
         model_upd = LGBMRegressor(n_estimators=500, first_metric_only=True)
-        model_upd.fit(X_train_upd, y_train,
-                      eval_set=[(X_test_upd, y_test)],
+        model_upd.fit(x_train_upd, y_train,
+                      eval_set=[(x_test_upd, y_test)],
                       eval_metric=['auc'],
                       early_stopping_rounds=10,
                       verbose=0)
-        top_features_ = pd.DataFrame(
-            [X_train_upd.columns,
-             model_upd.feature_importances_]).T.sort_values(1, ascending=False)
+        top_features_upd = pd.DataFrame([x_train_upd.columns, model_upd.feature_importances_]).T.sort_values(
+            1, ascending=False)
         return {
             'model': model_upd,
-            'features': top_features_,
+            'features': top_features_upd,
         }
 
     def get_all_models(self):
@@ -78,32 +74,6 @@ class Sample:
             logging.info(f'Model for {cell_type_} is generated')
 
 
-def draw_net(G, gene=False, layout='2', font_size=8):
-    b = nx.betweenness_centrality(G)
-    labels = {}
-    for node in G.nodes():
-        if node:
-            if node in gene:
-                labels[node] = node
-        else:
-            if G.degree[node] >= 1 or b[node] >= 0.08:
-                labels[node] = node
-    layouts = {'1': nx.spring_layout(G),
-               '2': nx.kamada_kawai_layout(G),
-               '3': nx.shell_layout(G)}
-
-    d = dict(G.degree)
-    pos = layouts[layout]
-    nx.draw(G,
-            pos=pos,
-            with_labels=False,
-            nodelist=d.keys(),
-            node_size=1500, alpha=0.7, node_color='lightblue', style='dashed', width=0.5)
-    nx.draw_networkx_labels(G, pos, labels, font_size=font_size, font_color='black')
-    plt.margins(x=0.4, y=0.4)
-    plt.show()
-
-
 class EvoManager:
     def __init__(self, *args):
         self.samples = dict(zip(list(range(len(args))), args))
@@ -135,8 +105,7 @@ class EvoManager:
         df_ = pd.DataFrame()
         for i in [x for x in df.columns if 'cluster' not in x]:
             df_[i] = list(df.groupby('clusters', as_index=False)[i].mean()[i])
-        df_.index = list(
-            df.groupby('clusters', as_index=False)[i].mean()['clusters'])
+        df_.index = list(df.groupby('clusters', as_index=False)[i].mean()['clusters'])
         return df_
 
     def between_two(self, sample1, sample2):
@@ -195,8 +164,10 @@ class EvoManager:
         ])
 
     @staticmethod
-    def get_shortest_paths(df, in_, out_, k=10):
-        network = nx.from_pandas_edgelist(df, 'Gene', 'Cell_type', ['Importance'])
+    def get_shortest_paths(network, in_, out_, k=10):
+        """
+        Generate K number of the shortest paths between selected nodes
+        """
         x = nx.shortest_simple_paths(network, in_, out_)
         for counter, path in enumerate(x):
             yield path
@@ -204,19 +175,25 @@ class EvoManager:
                 break
 
     def generate_comparison_df(self):
+        """
+        :return: confusion matrix for two samples
+        """
         return pd.concat([pd.concat([self.predictions['0_0'].apply(zscore),
                                      self.predictions['0_1'].apply(zscore)]),
                           pd.concat([self.predictions['1_0'].apply(zscore),
                                      self.predictions['1_1'].apply(zscore)])], axis=1)
 
     def cluster_map(self, visible_legend=False, c_map="viridis"):
+        """
+        Draw a clustermap for df.T (clustering by "how cell types were predicted with all the models")
+        """
         res = sns.clustermap(self.cell_types_similarity.T.corr(), cmap=c_map, yticklabels=True, figsize=(10, 10))
         res.cax.set_visible(visible_legend)
         plt.show()
 
     def get_closest_cell_types(self, cell_type, type_='matrix'):
         """
-        Returns sorted list of closest celltypes
+        Returns sorted list of closest cell types
         """
         assert type_ in ['model', 'matrix']
         if type_ == 'model':
@@ -224,40 +201,84 @@ class EvoManager:
         else:
             return list(self.cell_types_similarity.T.corr().sort_values(cell_type, ascending=False).index)
 
-    def check_(self, closest_cell_types, short_path):
-        for i in [x for x in short_path if x in list(self.network['Cell_type'].unique())]:
+    def check_(self, closest_cell_types, shortest_path):
+        for i in [x for x in shortest_path if x in list(self.network['Cell_type'].unique())]:
             if i not in list(set(closest_cell_types) & set(list(self.network['Cell_type'].unique()))):
                 return False
         return True
 
     @staticmethod
-    def write_connections(in_, out_, raw_lists_for_debug, short_path_):
-        raw_lists_for_debug.append(short_path_)
-        for j, k in enumerate(short_path_):
+    def write_connections(in_, out_, raw_lists_for_debug, shortest_path_):
+        """
+        Transforms the shortest paths into two lists of In and Out for future network df
+        """
+        raw_lists_for_debug.append(shortest_path_)
+        for j, k in enumerate(shortest_path_):
             try:
-                out_.append(short_path_[j + 1])
-                in_.append(short_path_[j])
+                out_.append(shortest_path_[j + 1])
+                in_.append(shortest_path_[j])
             except IndexError:
                 continue
 
-    def generate_cell_type_network(self, cluster1, cluster2, number_of_short_paths=100,
+    def generate_cell_type_network(self, cluster1, cluster2, number_of_shortest_paths=100,
                                    minimal_number_of_nodes=3, get_only_direct=False, closest_clusters=10, net=True):
-        in_, out_, raw_lists_for_debug = [], [], []
-        subnet = pd.DataFrame()
-        for short_path_ in [
-            x for x in self.get_shortest_paths(self.network,
-                                               cluster1, cluster2, number_of_short_paths)
+        """
+        Generate a subnetwork of the nodes and edges between two selected nodes
+        :param cluster1: in cluster
+        :param cluster2: out cluster
+        :param number_of_shortest_paths: how many shortest paths you do consider
+        :param minimal_number_of_nodes: minimal number of the nodes in the subnetwork
+        :param get_only_direct: generates subnetwork with len(shortestpaths)==3
+        :param closest_clusters: how many close cell types you do want to incorporate into the subnetwork
+        :param net: for debugging
+        """
+        in_, out_, raw_lists_for_debug, subnet = [], [], [], pd.DataFrame()
+        graph = nx.from_pandas_edgelist(self.network, 'Gene', 'Cell_type', ['Importance'])
+        for shortest_path_ in [
+            x for x in self.get_shortest_paths(graph,
+                                               cluster1, cluster2, number_of_shortest_paths)
         ]:
             if get_only_direct:
-                if len(short_path_) == minimal_number_of_nodes and self.check_(
-                        self.get_closest_cell_types(cluster2)[:closest_clusters], short_path_):
-                    self.write_connections(in_, out_, raw_lists_for_debug, short_path_)
-            elif len(short_path_) >= minimal_number_of_nodes and self.check_(
-                    self.get_closest_cell_types(cluster2)[:closest_clusters], short_path_):
-                self.write_connections(in_, out_, raw_lists_for_debug, short_path_)
+                if len(shortest_path_) == minimal_number_of_nodes and self.check_(
+                        self.get_closest_cell_types(cluster2)[:closest_clusters], shortest_path_):
+                    self.write_connections(in_, out_, raw_lists_for_debug, shortest_path_)
+            elif len(shortest_path_) >= minimal_number_of_nodes and self.check_(
+                    self.get_closest_cell_types(cluster2)[:closest_clusters], shortest_path_):
+                self.write_connections(in_, out_, raw_lists_for_debug, shortest_path_)
         subnet['in'], subnet['out'] = in_, out_
-        graph = nx.from_pandas_edgelist(subnet, 'in', 'out')
+        subnet['importance'] = [graph.get_edge_data(row['in'], row['out'])['Importance']
+                                for i, row in subnet.iterrows()]
+        subgraph = nx.from_pandas_edgelist(subnet, 'in', 'out', ['Importance'])
         if net:
-            return graph
+            return subgraph
         else:
-            return raw_lists_for_debug
+            return subnet
+
+
+def draw_net(graph, gene=False, layout='2', font_size=8):
+    """
+    Draw a graph using networkx (with gene labeling)
+    """
+    b = nx.betweenness_centrality(graph)
+    labels = {}
+    for node in graph.nodes():
+        if node:
+            if node in gene:
+                labels[node] = node
+        else:
+            if graph.degree[node] >= 1 or b[node] >= 0.08:
+                labels[node] = node
+    layouts = {'1': nx.spring_layout(graph),
+               '2': nx.kamada_kawai_layout(graph),
+               '3': nx.shell_layout(graph)}
+
+    d = dict(graph.degree)
+    pos = layouts[layout]
+    nx.draw(graph,
+            pos=pos,
+            with_labels=False,
+            nodelist=d.keys(),
+            node_size=1500, alpha=0.7, node_color='lightblue', style='dashed', width=0.5)
+    nx.draw_networkx_labels(graph, pos, labels, font_size=font_size, font_color='black')
+    plt.margins(x=0.4, y=0.4)
+    plt.show()
